//+------------------------------------------------------------------+
//|                    Isakas Ashi I Expert Advisor for MetaTrader 4 |
//|                                                     version 1.21 |
//|                            Copyright  2006-2008 Kuskus Osentogg |
//|                                     EA Conversion by Azmel Ainul |
//+------------------------------------------------------------------+
//| VERSION HISTORY                                                  |
//| 1.0  - Initial release.                                          |
//| 1.1  - Heiken Ashi read on candle close. Triple TF option.       |
//| 1.2  - All indicators read on candle close.                      |
//| 1.21 - Uses Time[] function to determine candle close.           |
//+------------------------------------------------------------------+

//EXTERNAL VARIABLES
extern int    Magic               = 748036;
extern bool   CompoundGrowth      = false;
extern double CompoundIndex       = 2.8;
extern double Lots                = 0.01;
extern int    TakeProfit          = 0;
extern bool   AutoTakeProfit      = false;
extern double AutoTakeProfitIndex = 1;
extern int    StopLoss            = 0;
extern bool   AutoStopLoss        = false;
extern int    AutoStopLossMin     = 30;
extern bool   CloseOnOpposite     = true;
extern bool   TripleHeikenAshi    = false;
extern int    MaxBarstocount      = 100;

//FOR ADMIN USE ONLY
bool   Admin;

//INTERNAL VARIABLES
double CandleTime;
double Leverage;
double LotPrice;
double PipValue;
double LotsMin;
double CapitalMin;
double DP;
double TP;
double SL;
double SafetyFactor;
int    i;
string line2;
double KS0;
double KS1;
string KSSignal;
double HAK0.M30;
double HAK1.M30;
double HAK0.H1;
double HAK1.H1;
double HAK0.H4;
double HAK1.H4;
double HS0;
double HS1;
string HSSignal;
string HAKSignal;
double EMA4;
double EMA24;
string EMASignal;
double LastHigh;
double LastLow;
string Status;
int    ticket;
int    slippage=5;

//INITIALISATION
int init()
{
   Status="StartUp";

   LotsMin=MarketInfo(Symbol(),MODE_MINLOT);
   switch(LotsMin)
   {
      case 1      : DP=0; break;
      case 0.1    : DP=1; break;
      case 0.01   : DP=2; break;
      case 0.001  : DP=3; break;
      case 0.0001 : DP=4; break;
      case 0.00001: DP=5; break;
   }
 
   CandleTime=Time[0];   

   return(0);
}

//MAIN ROUTINE
int start()
{
   //COMPOUND GROWTH
   if(CompoundGrowth)
   {
      LotPrice=MarketInfo(Symbol(),MODE_LOTSIZE);
      Lots=MathMax(LotsMin,NormalizeDouble(CompoundIndex*AccountBalance()/LotPrice,DP));
   }
   
   if(CandleTime!=Time[0])
   {
      //KUSKUS STOP INDICATOR (LAST HIGH AND LOW)
      LastHigh=0;
      LastLow=0;
      i=0;
      KSSignal="None";
      while(LastHigh==0 || LastLow==0)
      {
         KS0=iCustom(NULL,0,"KUskUs_stop",6,0,i);
         KS1=iCustom(NULL,0,"KUskUs_stop",6,1,i);
         if(LastHigh==0 && KS0>0 && KS0<1000)
         {
            if(KSSignal=="None")
            {
               KSSignal="Short";
            }
            if(KS0-Ask>AutoStopLossMin*Point)
            {
               LastHigh=KS0;
            }
         }
         if(LastLow==0 && KS1>0 && KS1<1000)
         {
            if(KSSignal=="None")
            {
               KSSignal="Long";
            }
            if(Ask-KS1>AutoStopLossMin*Point)
            {
               LastLow=KS1;
            }
         }
         
//----------------------
      if (i==MaxBarstocount){
         if (LastHigh==0){
            LastHigh=MathMax(Ask+AutoStopLossMin*Point,High[iHighest(NULL,0,MODE_HIGH,MaxBarstocount,1)]);
         }   
         if (LastLow==0){
            LastLow=MathMin(Ask-AutoStopLossMin*Point,Low[iLowest(NULL,0,MODE_LOW,MaxBarstocount,1)]);
         }   
      }
//-----------------------
         
         i++;
      }
   
      //HEIKEN ASHI KUSKUS INDICATOR
      if(TripleHeikenAshi)
      {
         HAK0.M30=iCustom(NULL,PERIOD_M30,"Heiken_Ashi_kuskus",1,40,0,1);
         HAK1.M30=iCustom(NULL,PERIOD_M30,"Heiken_Ashi_kuskus",1,40,1,1);
         HAK0.H1 =iCustom(NULL,PERIOD_H1 ,"Heiken_Ashi_kuskus",1,40,0,1);
         HAK1.H1 =iCustom(NULL,PERIOD_H1 ,"Heiken_Ashi_kuskus",1,40,1,1);
         HAK0.H4 =iCustom(NULL,PERIOD_H4 ,"Heiken_Ashi_kuskus",1,40,0,1);
         HAK1.H4 =iCustom(NULL,PERIOD_H4 ,"Heiken_Ashi_kuskus",1,40,1,1);
         if(HAK0.M30>HAK1.M30 && HAK0.H1>HAK1.H1 && HAK0.H4>HAK1.H4)
         {
            HAKSignal="Short";
         }
         if(HAK0.M30<HAK1.M30 && HAK0.H1<HAK1.H1 && HAK0.H4<HAK1.H4)
         {
            HAKSignal="Long";
         }
      }
      else
      {
         HAK0.H1 =iCustom(NULL,0,"Heiken_Ashi_kuskus",1,40,0,1);
         HAK1.H1 =iCustom(NULL,0,"Heiken_Ashi_kuskus",1,40,1,1);
         if(HAK0.H1>HAK1.H1)
         {
            HAKSignal="Short";
         }
         if(HAK0.H1<HAK0.H1)
         {
            HAKSignal="Long";
         }
      }

      //HIST STEPMA STOCH INDICATOR
      HS0=iCustom(NULL,0,"Hist_StepMA_Stoch_KV1_Ex_03",10,1,0,500,0,1);
      HS1=iCustom(NULL,0,"Hist_StepMA_Stoch_KV1_Ex_03",10,1,0,500,1,1);
      HSSignal="None";
      if(HS0!=0 && HS1==0)
      {
         HSSignal="Short";
      }
      if(HS0==0 && HS1!=0)
      {
         HSSignal="Long";
      }
   
      //EMA4 AND EMA24 INDICATORS
      EMA4= iMA(NULL,0, 4,0,MODE_EMA,PRICE_CLOSE,1);
      EMA24=iMA(NULL,0,24,0,MODE_EMA,PRICE_OPEN ,1);
      if(EMA4<EMA24)
      {
         EMASignal="Short";
      }
      if(EMA4>EMA24)
      {
         EMASignal="Long";
      }
      
      CandleTime=Time[0];
   }
   
   //CLOSE ON OPPOSITE SIGNAL
   if(CloseOnOpposite)
   {
      if(KSSignal=="Long")
      {
         CloseAllShorts();
      }
      if(KSSignal=="Short")
      {
         CloseAllLongs();
      }
   }

   //LONG SIGNAL
   if(KSSignal=="Long" && HAKSignal=="Long" && HSSignal=="Long" && EMASignal=="Long" && Status=="Ready")
   {
      if(AutoTakeProfit)
      {
         TakeProfit=AutoTakeProfitIndex*(LastHigh-Ask)/Point;
      }
      if(TakeProfit==0)
      {
         TP=0;
      }
      else
      {
         TP=Ask+TakeProfit*Point;
      }
      if(AutoStopLoss)
      {
         StopLoss=(Ask-LastLow)/Point;
         if(AutoStopLossMin>0 && AutoStopLossMin>StopLoss)
         {
            StopLoss=AutoStopLossMin;
         }
      }
      if(StopLoss==0)
      {
         SL=0;
      }
      else
      {
         SL=Ask-StopLoss*Point;
      }
      ticket=OrderSend(Symbol(),OP_BUY,Lots,Ask,slippage,SL,TP,"",Magic,0,Blue);
      if(ticket!=-1)
      {
         Status="Long Active";
      }
   }
   
   //SHORT SIGNAL   
   if(KSSignal=="Short" && HAKSignal=="Short" && HSSignal=="Short" && EMASignal=="Short" && Status=="Ready")
   {
      if(AutoTakeProfit)
      {
         TakeProfit=AutoTakeProfitIndex*(Bid-LastLow)/Point;
      }
      if(TakeProfit==0)
      {
         TP=0;
      }
      else
      {
         TP=Bid-TakeProfit*Point;
      }
      if(AutoStopLoss)
      {
         StopLoss=(LastHigh-Bid)/Point;
         if(AutoStopLossMin>0 && AutoStopLossMin>StopLoss)
         {
            StopLoss=AutoStopLossMin;
         }
      }
      if(StopLoss==0)
      {
         SL=0;
      }
      else
      {
         SL=Bid+StopLoss*Point;
      }
      ticket=OrderSend(Symbol(),OP_SELL,Lots,Bid,slippage,SL,TP,"",Magic,0,Red);
      if(ticket!=-1)
      {
         Status="Short Active";
      }
   }
   
   //READY SIGNAL
   if(Status=="Long Active" && KSSignal=="Short")
   {
      Status="Ready";
   }
   if(Status=="Short Active" && KSSignal=="Long")
   {
      Status="Ready";
   }
   
   //STARTUP DELAY
   if(Status=="StartUp")
   {
      if((KSSignal=="Short" && HAKSignal=="Short" && HSSignal=="Short" && EMASignal=="Short") || (KSSignal=="Long" && HAKSignal=="Long" && HSSignal=="Long" && EMASignal=="Long"))
      {
         Status="StartUp";
      }
      else
      {
         Status="Ready";
      }
   }
   
   //ON SCREEN COMMENTS
   if(Admin)
   {
      line2=StringConcatenate("Last High = " ,LastHigh ,"\n"
                             ,"Last Low = "  ,LastLow  ,"\n"
                             ,"KS Signal = " ,KSSignal ,"\n"
                             ,"HAK Signal = ",HAKSignal,"\n"
                             ,"HS Signal = " ,HSSignal ,"\n"
                             ,"EMA Signal = ",EMASignal,"\n"
                             ,"Status = "    ,Status
                             );
   }
   else
   {
      line2="";
   }
   Comment("\n","Isakas Ashi I EA ver 1.21"             ,"\n"
               ,"Copyright  2006-2008 Kuskus Osentogg","\n"
               ,"EA Conversion by Azmel Ainul"         ,"\n\n"
               ,line2);
   return(0);
}
          
int CloseAllLongs()
{
   for(i=0;i<OrdersTotal();i++)
   {
      OrderSelect(i,SELECT_BY_POS,MODE_TRADES);
      if(OrderSymbol()==Symbol() && OrderMagicNumber()==Magic && OrderType()==OP_BUY)
      {
         OrderClose(OrderTicket(),OrderLots(),Bid,slippage,CLR_NONE);
      }
   }
   return(0);
}

int CloseAllShorts()
{
   for(i=0;i<OrdersTotal();i++)
   {
      OrderSelect(i,SELECT_BY_POS,MODE_TRADES);
      if(OrderSymbol()==Symbol() && OrderMagicNumber()==Magic && OrderType()==OP_SELL)
      {
         OrderClose(OrderTicket(),OrderLots(),Ask,slippage,CLR_NONE);
      }
   }
   return(0);
}